#~ '(C) Links System Software 2009
#~ 'Feedback information
#~ 'www: http://lss2008.livejournal.com/
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru
#~ 'icq: 328-958-369

#~ sublend.rb ver. 1.5 16-Aug-09
#~ Little plug-in, which creates interpolated lines between two existing ones

#~ How to use:
#~ 1)	put sublend.rb to Google SketchUp 6\Plugins folder
#~ 2)	start SketchUp 6
#~ 3)	draw 2 curves then select them
#~ 4)	choose Plugins>> Blend command from the main menu
#~ 5)	Blend Settings input box appears, specify "Steps count" (how many intermediate lines/faces you want to generate)
#~      and click Ok

require 'sketchup.rb'

class Lss_SU_Blend
    
    def initialize (steps=10)
      @steps=steps
    end
    
    def blend(steps,draw_surf_flag, draw_tracks_flag, blend_skin_color)
      tracks_group = Sketchup.active_model.entities.add_group
      mesh_group = Sketchup.active_model.entities.add_group
      contours_group = Sketchup.active_model.entities.add_group
      @steps=steps
      @model=Sketchup.active_model
      @selection=@model.selection
      @selected_type=@selection[0].typename
      entities = Sketchup.active_model.entities
      points1=[]
      points2=[]
      case  @selected_type
        when "Edge"
          edges1=@selection[0].all_connected
          edges2=@selection[0].all_connected
          ind=0
          while edges2==edges1
            edges2=@selection[ind].all_connected
            ind +=1
          end
          if not edges1[0].curve.nil?
            ####################
            edges1_cnt=edges1.length  # need to find out if the curve is closed
            edges2_cnt=edges2.length  #
            ####################
            points1=edges1[0].curve.vertices 
            points2=edges2[0].curve.vertices
          else
            #it is the case when 2 one segment edges were selected
            points1=[edges1[0].start,edges1[0].end]
            points2=[edges2[0].start,edges2[0].end]
          end
        when "Face"
          face1=@selection[0]
          face2=@selection[1]
          points1=face1.vertices
          points2=face2.vertices
          
          mat1=face1.material
          mat2=face2.material
          bk_mat1=face1.back_material
          bk_mat2=face2.back_material
          ######################
          col1=mat1.color if mat1
          col2=mat2.color if mat2
          bk_col1=bk_mat1.color if bk_mat1
          bk_col2=bk_mat2.color if bk_mat2
          ######################
        end
     
      if points1.length>=points2.length
        max_pnts_cnt=points1.length
      else
        max_pnts_cnt=points2.length
      end
      step=0
      
      while step<=steps+1
        pnt_ind=0
        verts=[]
        while pnt_ind<max_pnts_cnt
          pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).round
          pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).round
          vert1=points1[pnt_ind1].position
          vert2=points2[pnt_ind2].position
          new_vert=[0,0,0]
          new_vert.x=vert1.x+step*(vert2.x-vert1.x)/(steps+1)
          new_vert.y=vert1.y+step*(vert2.y-vert1.y)/(steps+1)
          new_vert.z=vert1.z+step*(vert2.z-vert1.z)/(steps+1)
            verts += [new_vert]
          pnt_ind +=1
        end
        verts.uniq! #ver 1.4 added to avoid error messages kind of 'Duplicating points'
        #exception handling added (ver 1.4) to pass the exception when not all vertices of an intermediate face are not at the same plane
        #when exception occures, script adds curves
        if @selected_type=="Face"
          clr_rat2=255*step/steps
          clr_rat1=(255-clr_rat2)
          #######################
          @rd=col2.red*clr_rat2/255+col1.red*clr_rat1/255 if col1 and col2
          @grn=col2.green*clr_rat2/255+col1.green*clr_rat1/255 if col1 and col2
          @bl=col2.blue*clr_rat2/255+col1.blue*clr_rat1/255 if col1 and col2
          @alp=mat2.alpha*clr_rat2/255+mat1.alpha*clr_rat1/255 if mat1 and mat2
          #~ @blended_mat=Sketchup.active_model.materials.add "lss_blended_mat #{step}"
          @blended_mat=[@rd, @grn, @bl, @alp]
          #######################
          @rd1=bk_col2.red*clr_rat2/255+bk_col1.red*clr_rat1/255 if col1 and col2
          @grn1=bk_col2.green*clr_rat2/255+bk_col1.green*clr_rat1/255 if col1 and col2
          @bl1=bk_col2.blue*clr_rat2/255+bk_col1.blue*clr_rat1/255 if col1 and col2
          @alp1=bk_mat2.alpha*clr_rat2/255+bk_mat1.alpha*clr_rat1/255 if mat1 and mat2
          #~ @blended_back_mat=Sketchup.active_model.materials.add "lss_back_blended_mat #{step}"
          @blended_back_mat=[@rd1, @grn1, @bl1, @alp1]
          #######################
          if not draw_surf_flag=="Surface only (suppress contours)"
            begin
              new_face=contours_group.entities.add_face verts
              new_face.material=@blended_mat
              new_face.back_material=@blended_back_mat
            rescue
              new_curve=contours_group.entities.add_curve verts #ver 1.4 added to draw at least a curve if it is not possible to draw face (when all vertices are not on the same plane)
              contours_group.entities.add_edges new_curve[0].start, new_curve[new_curve.length-1].end
            end
          end
        end
        if (@selected_type=="Edge" and not draw_surf_flag=="Surface only (suppress contours)") 
          new_curve=contours_group.entities.add_curve verts
          if new_curve.length<edges1_cnt or new_curve.length<edges2_cnt #check if the initial curve was closed or no
            contours_group.entities.add_edges new_curve[0].start, new_curve[new_curve.length-1].end #close it if yes
          end
        end
        step +=1
        if (draw_surf_flag=="Yes" or draw_surf_flag=="Surface only (suppress contours)") and step<=steps+1
          pnt_ind=0
          verts1=[]
          while pnt_ind<max_pnts_cnt
            pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).round
            pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).round
            vert1=points1[pnt_ind1].position
            vert2=points2[pnt_ind2].position
            new_vert=[0,0,0]
            new_vert.x=vert1.x+step*(vert2.x-vert1.x)/(steps+1)
            new_vert.y=vert1.y+step*(vert2.y-vert1.y)/(steps+1)
            new_vert.z=vert1.z+step*(vert2.z-vert1.z)/(steps+1)
            verts1 += [new_vert] 
            pnt_ind +=1
          end
          draw_surf verts,verts1,mesh_group,@selected_type
        end
        
        if draw_tracks_flag=="Yes" and step<=steps+1
          pnt_ind=0
          verts1=[]
          while pnt_ind<max_pnts_cnt
            pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).round
            pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).round
            vert1=points1[pnt_ind1].position
            vert2=points2[pnt_ind2].position
            new_vert=[0,0,0]
            new_vert.x=vert1.x+step*(vert2.x-vert1.x)/(steps+1)
            new_vert.y=vert1.y+step*(vert2.y-vert1.y)/(steps+1)
            new_vert.z=vert1.z+step*(vert2.z-vert1.z)/(steps+1)
            verts1 += [new_vert] 
            pnt_ind +=1
          end
          draw_vert_tracks verts,verts1,tracks_group
        end
      end
      mesh=Geom::PolygonMesh.new
      mesh_group.entities.each{|face|
        pt1=face.vertices[0].position if face.vertices[0]
        pt2=face.vertices[1].position if face.vertices[1]
        pt3=face.vertices[2].position if face.vertices[2]
        mesh.add_polygon pt1,pt2,pt3 if (pt1 and pt2 and pt3)
      }
      if blend_skin_color=="No"
        mesh_group.entities.clear!
        mesh_group.entities.add_faces_from_mesh mesh, 12
      end
    end
    
    def draw_surf (points1, points2,group, face_flag)
      if points1.length>=points2.length
        max_pnts_cnt=points1.length
      else
        max_pnts_cnt=points2.length
      end
      pnt_ind=-1 if face_flag=="Face" #because the face is closed (first vetex coincides with last one)
      pnt_ind=0 if not face_flag=="Face"
      while pnt_ind<max_pnts_cnt
        pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).truncate
        pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).truncate
        vert1=points1[pnt_ind1]
        vert2=points2[pnt_ind2]
        pnt_ind3=((pnt_ind+1)*points1.length/max_pnts_cnt).truncate
        pnt_ind4=((pnt_ind+1)*points2.length/max_pnts_cnt).truncate
        vert3=points1[pnt_ind3]
        vert4=points2[pnt_ind4]
        #exception handling added for complex curves processing (complex geometry generates some faces with duplicating vertices and Skethup can not add them)
        begin
          if (not vert1.nil? and not vert2.nil? and not vert3.nil? and not vert1==vert3)
            mesh_face1=group.entities.add_face vert1,vert2,vert3 
            mesh_face1.material=@blended_mat if not @blended_mat[0].nil?
            mesh_face1.back_material=@blended_back_mat if not @blended_back_mat[0].nil?
          end
          if (not vert2.nil? and not vert4.nil? and not vert3.nil? and not vert2==vert4)
            mesh_face2=group.entities.add_face vert3,vert4,vert2
            mesh_face2.material=@blended_mat if not @blended_mat[0].nil?
            mesh_face2.back_material=@blended_back_mat if not @blended_back_mat[0].nil?
          end
        rescue
          puts "Some 'bad' faces were ignored."
        end
        pnt_ind +=1
      end
    end
    
    def draw_vert_tracks (pnts1, pnts2,group) 
      points1=pnts1.uniq
      points2=pnts2.uniq
      if points1.length>=points2.length
        max_pnts_cnt=points1.length
      else
        max_pnts_cnt=points2.length
      end
      pnt_ind=0
      while pnt_ind<max_pnts_cnt
        pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).truncate
        pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).truncate
        vert1=points1[pnt_ind1]
        vert2=points2[pnt_ind2]
        #~ begin
          group.entities.add_edges vert1,vert2
        #~ rescue
          #~ puts "Some 'bad' edges were ignored."
        #~ end
        pnt_ind +=1
      end
    end
  end
  
tool_menu = UI.menu "Plugins" 
 tool_menu.add_item("Blend") {
  selection=Sketchup.active_model.selection
  if selection.count<2
    UI.messagebox "It is necessary to select two curves or two faces or two lines."
    return
  end
  if selection[0].typename=="Edge"
    edges=selection[0].all_connected
    if edges.length==selection.count
      UI.messagebox "It is necessary to select two curves"
      return
    end
  end
  model = Sketchup.active_model 

  
  prompts = ["Steps count: ", "Connect steps with surface ", "Draw vertices tracks ", "Blend 'skin' color"]
  defaults= [10, "No", "No", "No"]
  list = ["", "No|Yes|Surface only (suppress contours)", "No|Yes", "No|Yes"]
  settings=UI.inputbox prompts, defaults, list, "Blend Settings"
  lss_blend=Lss_SU_Blend.new
  status = model.start_operation "Blend"
  lss_blend.blend settings[0], settings[1], settings[2], settings[3]
  
  status = model.commit_operation
}